require 'date'
require 'roxml'
 

module MastercardPspPayment
  # This class contains methods to get the consumer's address details.
  class Address
    include ROXML
    
    xml_name "Address"
    
    # @!attribute city
    # @return [String] the cardholder's city.
    xml_accessor :city, :from =>"city"
   
    # @!attribute country
    # @return [String] the cardholder's country as defined by ISO 3166-1 alpha-2 digit country codes; for example, US is the United States, AU is Australia, CA is Canada, GB is the United Kingdom, and so on.
    xml_accessor :country, :from =>"country"
   
    # @!attribute subdivision
    # @return [String] the cardholder's country's subdivision as defined by ISO 3166-1 alpha-2 digit code; for example, US-VA is Virginia, US-OH is Ohio, and so on.
    xml_accessor :subdivision, :from =>"subdivision"
   
    # @!attribute line1
    # @return [String] the address in line 1 is used for the street number and the street name.
    xml_accessor :line1, :from =>"line1"
   
    # @!attribute line2
    # @return [String] the address in line 2 is used for the apartment number, suite number, and so on.
    xml_accessor :line2, :from =>"line2"
   
    # @!attribute line3
    # @return [String] the address in line 3 is used to enter the remaining address information if it does not fit in lines 1 and 2.
    xml_accessor :line3, :from =>"line3"
   
    # @!attribute line4
    # @return [String] the address in line 4 is used to enter the remaining address information if it does not fit in lines 1, 2 and 3.
    xml_accessor :line4, :from =>"line4"
   
    # @!attribute line5
    # @return [String] the address in line 5 is used to enter the remaining address information if it does not fit in lines 1,2,3 and 4.
    xml_accessor :line5, :from =>"line5"
   
    # @!attribute postal_code
    # @return [String] the postal code or zip code appended to the mailing address for the purpose of sorting mail.
    xml_accessor :postal_code, :from =>"postalCode"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :city => :city ,
        :country => :country ,
        :subdivision => :subdivision ,
        :line1 => :line1 ,
        :line2 => :line2 ,
        :line3 => :line3 ,
        :line4 => :line4 ,
        :line5 => :line5 ,
        :postal_code => :postalCode 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:city)
        self.city = attributes[:city]
      end
      
      if attributes.has_key?(:country)
        self.country = attributes[:country]
      end
      
      if attributes.has_key?(:subdivision)
        self.subdivision = attributes[:subdivision]
      end
      
      if attributes.has_key?(:line1)
        self.line1 = attributes[:line1]
      end
      
      if attributes.has_key?(:line2)
        self.line2 = attributes[:line2]
      end
      
      if attributes.has_key?(:line3)
        self.line3 = attributes[:line3]
      end
      
      if attributes.has_key?(:line4)
        self.line4 = attributes[:line4]
      end
      
      if attributes.has_key?(:line5)
        self.line5 = attributes[:line5]
      end
      
      if attributes.has_key?(:postal_code)
        self.postal_code = attributes[:postal_code]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          city == o.city &&
          country == o.country &&
          subdivision == o.subdivision &&
          line1 == o.line1 &&
          line2 == o.line2 &&
          line3 == o.line3 &&
          line4 == o.line4 &&
          line5 == o.line5 &&
          postal_code == o.postal_code
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [city, country, subdivision, line1, line2, line3, line4, line5, postal_code].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardPspPayment.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Address" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :city => 'String',
        :country => 'String',
        :subdivision => 'String',
        :line1 => 'String',
        :line2 => 'String',
        :line3 => 'String',
        :line4 => 'String',
        :line5 => 'String',
        :postal_code => 'String'
        
      }
    end
  end


end
