require 'date'
require 'roxml'
 

module MastercardPspPayment
  # This class contains methods to set different authentication options required during DSRP.
  class AuthenticationOptions
    include ROXML
    
    xml_name "AuthenticationOptions"
    
    # @!attribute authenticate_method
    # @return [String] the method used to authenticate the cardholder at checkout. Valid values are MERCHANT ONLY, 3DS and No Authentication.
    xml_accessor :authenticate_method, :from =>"authenticateMethod"
   
    # @!attribute card_enrollment_method
    # @return [String] the method by which the card was added to the wallet. Valid values are: Manual Direct Provisioned 3DS Manual NFC Tap.
    xml_accessor :card_enrollment_method, :from =>"cardEnrollmentMethod"
   
    # @!attribute c_avv
    # @return [String] the (CAVV) Cardholder Authentication Verification Value generated by card issuer upon successful authentication of the cardholder. This must be passed in the authorization message.
    xml_accessor :c_avv, :from =>"cAvv"
   
    # @!attribute eci_flag
    # @return [String] the Electronic commerce indicator (ECI) flag. Possible values are as follows: Mastercard: 00:No Authentication 01:Attempts (Card Issuer Liability) 02:Authenticated by ACS (Card Issuer Liability) 03:Maestro (MARP) 05:Risk Based Authentication (Issuer, not in use) 06:Risk Based Authentication (Merchant, not in use) Visa: 05:Authenticated (Card Issuer Liability) 06:Attempts (Card Issuer Liability) 07:No 3DS Authentication (Merchant Liability)
    xml_accessor :eci_flag, :from =>"eciFlag"
   
    # @!attribute master_card_assigned_id
    # @return [String] the value assigned by Mastercard and represents programs associated directly with Maestro cards. This field should be supplied in the authorization request by the merchant.
    xml_accessor :master_card_assigned_id, :from =>"masterCardAssignedID"
   
    # @!attribute pa_res_status
    # @return [String] the message formatted, digitally signed, and sent from the ACS (issuer) to the MPI providing the results of the issuer's Mastercard SecureCode/Verified by Visa cardholder authentication. Possible values are: Y-The card was successfully authenticated via 3-D Secure A-signifies that either (a) the transaction was successfully authenticated via a 3-D Secure attempts transaction or (b)the cardholder was prompted to activate 3-D Secure during shopping but declined (Visa). U-Authentication results were unavailable.
    xml_accessor :pa_res_status, :from =>"paResStatus"
   
    # @!attribute sc_enrollment_status
    # @return [String] the Mastercard SecureCode Enrollment Status. Indicates if the issuer of the card supports payer authentication for this card. Possible values are as follows: Y-The card is eligible for 3-D Secure authentication. N-The card is not eligible for 3-D Secure authentication. U-Lookup of the card's 3-D Secure eligibility status was either unavailable, or the card is inapplicable (for example, prepaid cards).
    xml_accessor :sc_enrollment_status, :from =>"scEnrollmentStatus"
   
    # @!attribute signature_verification
    # @return [String] the signature verification. Possible values are as follows: Y-Indicates that the signature of the PaRes has been validated successfully and the message contents can be trusted. N-Indicates that for a variety of reasons (tampering, certificate expiration, and so on) the PaRes could not be validated, and the result should not be trusted.
    xml_accessor :signature_verification, :from =>"signatureVerification"
   
    # @!attribute xid
    # @return [String] the transaction identifier resulting from authentication processing.
    xml_accessor :xid, :from =>"xid"
   
    # @!attribute aav
    # @return [String] the accountholder authentication value resulting from authentication processing.
    xml_accessor :aav, :from =>"aav"
   
    # @!attribute sli
    # @return [String] the security level indicator resulting from authentication processing.
    xml_accessor :sli, :from =>"sli"
   
    # @!attribute card_verification_status
    # @return [String] the card verification status resulting from authentication processing.
    xml_accessor :card_verification_status, :from =>"cardVerificationStatus"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :authenticate_method => :authenticateMethod ,
        :card_enrollment_method => :cardEnrollmentMethod ,
        :c_avv => :cAvv ,
        :eci_flag => :eciFlag ,
        :master_card_assigned_id => :masterCardAssignedID ,
        :pa_res_status => :paResStatus ,
        :sc_enrollment_status => :scEnrollmentStatus ,
        :signature_verification => :signatureVerification ,
        :xid => :xid ,
        :aav => :aav ,
        :sli => :sli ,
        :card_verification_status => :cardVerificationStatus 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:authenticate_method)
        self.authenticate_method = attributes[:authenticate_method]
      end
      
      if attributes.has_key?(:card_enrollment_method)
        self.card_enrollment_method = attributes[:card_enrollment_method]
      end
      
      if attributes.has_key?(:c_avv)
        self.c_avv = attributes[:c_avv]
      end
      
      if attributes.has_key?(:eci_flag)
        self.eci_flag = attributes[:eci_flag]
      end
      
      if attributes.has_key?(:master_card_assigned_id)
        self.master_card_assigned_id = attributes[:master_card_assigned_id]
      end
      
      if attributes.has_key?(:pa_res_status)
        self.pa_res_status = attributes[:pa_res_status]
      end
      
      if attributes.has_key?(:sc_enrollment_status)
        self.sc_enrollment_status = attributes[:sc_enrollment_status]
      end
      
      if attributes.has_key?(:signature_verification)
        self.signature_verification = attributes[:signature_verification]
      end
      
      if attributes.has_key?(:xid)
        self.xid = attributes[:xid]
      end
      
      if attributes.has_key?(:aav)
        self.aav = attributes[:aav]
      end
      
      if attributes.has_key?(:sli)
        self.sli = attributes[:sli]
      end
      
      if attributes.has_key?(:card_verification_status)
        self.card_verification_status = attributes[:card_verification_status]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          authenticate_method == o.authenticate_method &&
          card_enrollment_method == o.card_enrollment_method &&
          c_avv == o.c_avv &&
          eci_flag == o.eci_flag &&
          master_card_assigned_id == o.master_card_assigned_id &&
          pa_res_status == o.pa_res_status &&
          sc_enrollment_status == o.sc_enrollment_status &&
          signature_verification == o.signature_verification &&
          xid == o.xid &&
          aav == o.aav &&
          sli == o.sli &&
          card_verification_status == o.card_verification_status
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [authenticate_method, card_enrollment_method, c_avv, eci_flag, master_card_assigned_id, pa_res_status, sc_enrollment_status, signature_verification, xid, aav, sli, card_verification_status].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardPspPayment.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "AuthenticationOptions" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :authenticate_method => 'String',
        :card_enrollment_method => 'String',
        :c_avv => 'String',
        :eci_flag => 'String',
        :master_card_assigned_id => 'String',
        :pa_res_status => 'String',
        :sc_enrollment_status => 'String',
        :signature_verification => 'String',
        :xid => 'String',
        :aav => 'String',
        :sli => 'String',
        :card_verification_status => 'String'
        
      }
    end
  end


end
