require 'date'
require 'roxml'
require_relative '../../mastercard_psp_payment/models/address'
 

module MastercardPspPayment
  # This class contains Card details for the available cards.
  class Card
    include ROXML
    
    xml_name "Card"
    
    # @!attribute brand_id
    # @return [String] the card's brand id; for example, master for Mastercard.
    xml_accessor :brand_id, :from =>"brandId"
   
    # @!attribute brand_name
    # @return [String] the card's brand name; for example, Mastercard.
    xml_accessor :brand_name, :from =>"brandName"
   
    # @!attribute account_number
    # @return [String] the card number, also known as Primary Account Number (PAN).
    xml_accessor :account_number, :from =>"accountNumber"
   
    # @!attribute card_holder_name
    # @return [String] the cardholder's name.
    xml_accessor :card_holder_name, :from =>"cardHolderName"
   
    # @!attribute expiry_month
    # @return [Integer] the PAN expiration month, returned as a one or two digit integer. No leading zero is required for single digit months e.g. April is 4.
    xml_accessor :expiry_month, :from =>"expiryMonth"
   
    # @!attribute expiry_year
    # @return [Integer] the PAN expiration year, returned as a four digit integer.
    xml_accessor :expiry_year, :from =>"expiryYear"
   
    # @!attribute billing_address
    # @return [Address] the billing details.
    xml_accessor :billing_address, :from =>"billingAddress",:as => Address
   
    # @!attribute last_four
    # @return [String] the last four digit of actual card number.
    xml_accessor :last_four, :from =>"lastFour"
   
    # @!attribute card_type
    # @return [String] the card type (CREDIT/DEBIT) used in checkout. Available only for combo card transactions.
    xml_accessor :card_type, :from =>"cardType"
   
    # @!attribute cvc
    # @return [String] the card verification code.
    xml_accessor :cvc, :from =>"cvc"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :brand_id => :brandId ,
        :brand_name => :brandName ,
        :account_number => :accountNumber ,
        :card_holder_name => :cardHolderName ,
        :expiry_month => :expiryMonth ,
        :expiry_year => :expiryYear ,
        :billing_address => :billingAddress ,
        :last_four => :lastFour ,
        :card_type => :cardType ,
        :cvc => :cvc 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:brand_id)
        self.brand_id = attributes[:brand_id]
      end
      
      if attributes.has_key?(:brand_name)
        self.brand_name = attributes[:brand_name]
      end
      
      if attributes.has_key?(:account_number)
        self.account_number = attributes[:account_number]
      end
      
      if attributes.has_key?(:card_holder_name)
        self.card_holder_name = attributes[:card_holder_name]
      end
      
      if attributes.has_key?(:expiry_month)
        self.expiry_month = attributes[:expiry_month]
      end
      
      if attributes.has_key?(:expiry_year)
        self.expiry_year = attributes[:expiry_year]
      end
      
      if attributes.has_key?(:billing_address)
        self.billing_address = attributes[:billing_address]
      end
      
      if attributes.has_key?(:last_four)
        self.last_four = attributes[:last_four]
      end
      
      if attributes.has_key?(:card_type)
        self.card_type = attributes[:card_type]
      end
      
      if attributes.has_key?(:cvc)
        self.cvc = attributes[:cvc]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          brand_id == o.brand_id &&
          brand_name == o.brand_name &&
          account_number == o.account_number &&
          card_holder_name == o.card_holder_name &&
          expiry_month == o.expiry_month &&
          expiry_year == o.expiry_year &&
          billing_address == o.billing_address &&
          last_four == o.last_four &&
          card_type == o.card_type &&
          cvc == o.cvc
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [brand_id, brand_name, account_number, card_holder_name, expiry_month, expiry_year, billing_address, last_four, card_type, cvc].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardPspPayment.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Card" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :brand_id => 'String',
        :brand_name => 'String',
        :account_number => 'String',
        :card_holder_name => 'String',
        :expiry_month => 'Integer',
        :expiry_year => 'Integer',
        :billing_address => 'Address',
        :last_four => 'String',
        :card_type => 'String',
        :cvc => 'String'
        
      }
    end
  end


end
