require 'date'
require 'roxml'
 

module MastercardPspPayment
  # This class contains the cryptogram generated by the consumer's Masterpass wallet.
  class Cryptogram
    include ROXML
    
    xml_name "Cryptogram"
    
    # @!attribute crypto_value
    # @return [String] the cryptogram generated by the consumer's Masterpass wallet.
    xml_accessor :crypto_value, :from =>"cryptoValue"
   
    # @!attribute crypto_type
    # @return [String] the type of cryptogram generated by the consumers Masterpass wallet. Masterpass passes the most secure selection (ICC) if the merchant or service provider has indicated they can accept both types (UCAF, ICC).
    xml_accessor :crypto_type, :from =>"cryptoType"
   
    # @!attribute unpredictable_number
    # @return [String] the unpredictable number. EMV quality random number generated by the merchant, service provider, or, if null, by Masterpass and Base64 encoded.
    xml_accessor :unpredictable_number, :from =>"unpredictableNumber"
   
    # @!attribute eci
    # @return [String] the electronic commerce indicator (ECI) value (DE 48 SE 42 position 3). Present only when crypto type is UCAF. For Mastercard brand cards, value is: 02 Authenticated by ACS (Card Issuer Liability)
    xml_accessor :eci, :from =>"eci"
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :crypto_value => :cryptoValue ,
        :crypto_type => :cryptoType ,
        :unpredictable_number => :unpredictableNumber ,
        :eci => :eci 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:crypto_value)
        self.crypto_value = attributes[:crypto_value]
      end
      
      if attributes.has_key?(:crypto_type)
        self.crypto_type = attributes[:crypto_type]
      end
      
      if attributes.has_key?(:unpredictable_number)
        self.unpredictable_number = attributes[:unpredictable_number]
      end
      
      if attributes.has_key?(:eci)
        self.eci = attributes[:eci]
      end
      
    end
    
 
	

    # Custom attribute writer method checking allowed values (enum).
    def crypto_type=(crypto_type)
      allowed_values = ["ICC", "UCAF", "TAVV"]
      if crypto_type && !allowed_values.include?(crypto_type)
        fail "invalid value for 'crypto_type', must be one of #{allowed_values}"
      end
      @crypto_type = crypto_type
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          crypto_value == o.crypto_value &&
          crypto_type == o.crypto_type &&
          unpredictable_number == o.unpredictable_number &&
          eci == o.eci
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [crypto_value, crypto_type, unpredictable_number, eci].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardPspPayment.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Cryptogram" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :crypto_value => 'String',
        :crypto_type => 'String',
        :unpredictable_number => 'String',
        :eci => 'String'
        
      }
    end
  end


end
