require 'date'
require 'roxml'
require_relative '../../mastercard_psp_payment/models/authentication_options'
require_relative '../../mastercard_psp_payment/models/card'
require_relative '../../mastercard_psp_payment/models/tokenization'
 

module MastercardPspPayment
  # This class contains various methods for to set different merchant initialization request parameters required for Merchant Initialization Service.
  class PaymentData
    include ROXML
    
    xml_name "PaymentData"
    
    # @!attribute card
    # @return [Card] the card details.
    xml_accessor :card, :from =>"card",:as => Card
   
    # @!attribute tokenization
    # @return [Tokenization] the tokenized card information.
    xml_accessor :tokenization, :from =>"tokenization",:as => Tokenization
   
    # @!attribute wallet_id
    # @return [String] the value which helps to identify origin wallet.
    xml_accessor :wallet_id, :from =>"walletId"
   
    # @!attribute authentication_options
    # @return [AuthenticationOptions] the tokenized card information.
    xml_accessor :authentication_options, :from =>"authenticationOptions",:as => AuthenticationOptions
   
    xml_accessor :external_data, :from =>"externalData", :as =>[]
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :card => :card ,
        :tokenization => :tokenization ,
        :wallet_id => :walletId ,
        :authentication_options => :authenticationOptions ,
        :external_data => :externalData 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:card)
        self.card = attributes[:card]
      end
      
      if attributes.has_key?(:tokenization)
        self.tokenization = attributes[:tokenization]
      end
      
      if attributes.has_key?(:wallet_id)
        self.wallet_id = attributes[:wallet_id]
      end
      
      if attributes.has_key?(:authentication_options)
        self.authentication_options = attributes[:authentication_options]
      end
      
      if attributes.has_key?(:external_data)
        self.external_data = attributes[:external_data]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          card == o.card &&
          tokenization == o.tokenization &&
          wallet_id == o.wallet_id &&
          authentication_options == o.authentication_options &&
          external_data == o.external_data
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [card, tokenization, wallet_id, authentication_options, external_data].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardPspPayment.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "PaymentData" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :card => 'Card',
        :tokenization => 'Tokenization',
        :wallet_id => 'String',
        :authentication_options => 'AuthenticationOptions',
        :external_data => 'Hash<String, Object>'
        
      }
    end
  end


end
