require 'date'
require 'roxml'
require_relative '../../mastercard_psp_payment/models/cryptogram'
 

module MastercardPspPayment
  # This class contains methods to get the tokenized card details.
  class Tokenization
    include ROXML
    
    xml_name "Tokenization"
    
    # @!attribute f_pan_suffix
    # @return [String] the last digits of card number or primary account number that identifies the card. Also known as Funding Primary Account Number. Normally suffix is the last four digits.
    xml_accessor :f_pan_suffix, :from =>"fPanSuffix"
   
    # @!attribute token_requestor_id
    # @return [String] the token request or id.
    xml_accessor :token_requestor_id, :from =>"tokenRequestorId"
   
    # @!attribute payment_account_reference
    # @return [String] the payment account reference.
    xml_accessor :payment_account_reference, :from =>"paymentAccountReference"
   
    # @!attribute source_system
    # @return [String] the source system. Currently, the only valid source system is: MDES.
    xml_accessor :source_system, :from =>"sourceSystem"
   
    # @!attribute cryptogram
    # @return [Cryptogram] the cryptogram generated by the consumer's Masterpass wallet.
    xml_accessor :cryptogram, :from =>"cryptogram",:as => Cryptogram
   

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :f_pan_suffix => :fPanSuffix ,
        :token_requestor_id => :tokenRequestorId ,
        :payment_account_reference => :paymentAccountReference ,
        :source_system => :sourceSystem ,
        :cryptogram => :cryptogram 
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes.has_key?(:f_pan_suffix)
        self.f_pan_suffix = attributes[:f_pan_suffix]
      end
      
      if attributes.has_key?(:token_requestor_id)
        self.token_requestor_id = attributes[:token_requestor_id]
      end
      
      if attributes.has_key?(:payment_account_reference)
        self.payment_account_reference = attributes[:payment_account_reference]
      end
      
      if attributes.has_key?(:source_system)
        self.source_system = attributes[:source_system]
      end
      
      if attributes.has_key?(:cryptogram)
        self.cryptogram = attributes[:cryptogram]
      end
      
    end
    
 
	

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          f_pan_suffix == o.f_pan_suffix &&
          token_requestor_id == o.token_requestor_id &&
          payment_account_reference == o.payment_account_reference &&
          source_system == o.source_system &&
          cryptogram == o.cryptogram
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [f_pan_suffix, token_requestor_id, payment_account_reference, source_system, cryptogram].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.datatype_map.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :Object
      	value
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k).to_sym] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = MastercardPspPayment.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash(include_root = false)
      attributes_hash = {} 
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      attributes_hash = include_root ? { "Tokenization" => hash } : hash  
      return attributes_hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end


    private
    def after_parse
      self.send(:remove_instance_variable, :@roxml_references) if defined? self.roxml_references
    end
    
    # Attribute datatype mapping.
    def self.datatype_map
      {
        :f_pan_suffix => 'String',
        :token_requestor_id => 'String',
        :payment_account_reference => 'String',
        :source_system => 'String',
        :cryptogram => 'Cryptogram'
        
      }
    end
  end


end
